/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#ifdef WIN_32
    #define  WIN_32_LEAN_AND_MEAN
    #include <windows.h>
#endif
#ifndef WIN_32
	#include "../config.h"
#endif

#include "./newtEngine/glUtil.h"
#include "./newtEngine/graphicOptions.h"
#include "keyboardOptions.h"
#include "fileManager.h"
#include "window.h"
#include "newtPuzzle.h"
#include "./newtEngine/scene.h"
#include "./newtEngine/shaderManager.h"

 
void checkSupportedExtensions() {
	VBOsupported = isExtensionAvailable("GL_ARB_vertex_buffer_object");
	//VBOsupported = false;	
	
	GLSLsupported = isExtensionAvailable("GL_ARB_shader_objects")&&
					isExtensionAvailable("GL_ARB_shading_language_100")&&
					isExtensionAvailable("GL_ARB_fragment_shader")&&
					isExtensionAvailable("GL_ARB_vertex_shader");
	
	FBOsupported =  isExtensionAvailable("GL_EXT_framebuffer_object")||
					isExtensionAvailable("GL_ARB_framebuffer_object");

	
	//printGLInfos();
	//printGLSLInfos();	
}


void initNewtPuzzle() {
	
	scene = new Scene();

	if(GLSLsupported) {
		shaderMana = new ShaderManager();
	} else {
		shaderMana = NULL;
	}
			
	newt = new NewtPuzzle(); 
	
}



int main( int argc, char *argv[] ) {

    /* Print copyright notice */
	std::cout << "Newt-Puzzle " << VERSION << " --  http://newtpuzzle.tuxfamily.org" << std::endl;
	std::cout << "(c) 2008-2009 Aude Jaulent <newtpuzzle@lists.tuxfamily.org>" << std::endl;
	std::cout << "Newt-Puzzle comes with ABSOLUTELY NO WARRANTY. This is free software,\n"
				 "and you are welcome to redistribute it under certain conditions.\n"
				 "See http://www.gnu.org/copyleft/gpl.html for details.\n" << std::endl;	
	
	
	srand( time(NULL) );
	
	fileMana = new FileManager();
	fileMana->initDir();
	
	generalGraphics = new GeneralGraphicOptions();	
	generalKeyboardOptions = new GeneralKeyboardOptions ();
	
	try {
		window_init(&argc, argv,"Newt-Puzzle", "Newt-Puzzle");
		checkSupportedExtensions();
		
		initNewtPuzzle ();
				
		window_enable_key_repeat(false);
		window_event(true);

		return EXIT_SUCCESS;
		
	} catch (std::exception &e) {
		std::cerr << e.what() << std::endl;
		std::cerr << "Aborting Newt-Puzzle :(" << std::endl;
		return EXIT_FAILURE;	
    }
}
