/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "fog.h"


Fog::Fog(GLint mode, GLfloat density, GLfloat start, GLfloat end, int numColor) :
	mMode(mode), mDensity(density), mStart(start), mEnd(end) {
	setColor(numColor);
}

Fog::Fog(GLint mode, GLfloat density, GLfloat start, GLfloat end, int numColor, float coeff) :
	mMode(mode), mDensity(density), mStart(start), mEnd(end) {
	setColor(numColor, coeff);
}

Fog::~Fog() {
}

void Fog::setColor(int numColor) {
	mColor = Color(numColor, -0.8, 0.);
}

void Fog::setColor(int numColor, float coeff) {
	mColor = Color(numColor, coeff, 0.);	
}
	
Color Fog::getColor() {
	return mColor;
}

	 
void Fog::activate() {	
	glEnable(GL_FOG);
	glFogi(GL_FOG_MODE, mMode);
	glFogfv(GL_FOG_COLOR, mColor);
	glFogf(GL_FOG_DENSITY, mDensity);
	glFogf(GL_FOG_START, mStart);
	glFogf(GL_FOG_END, mEnd);		
}	

void Fog::release() {
	glDisable(GL_FOG);	
}
