/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _SHADERMANAGER_H_
#define _SHADERMANAGER_H_

#include <map>
#include <set>

#include "glUtil.h"
#include "shaderLoader.h"
#include "shaderWriter.h"
#include "shader.h"
#include "materialShader.h"
#include "perFragLightShader.h"
#include "skyShader.h"

#include "../fileManager.h"


enum shaderType {FIXED_PIPELINE, TOON, PER_FRAG_LIGHT, SKY};


class ShaderManager
{
public:
	ShaderManager();
	~ShaderManager();

	shaderType getCurrentShaderType();
		
	int getCurrentLightKey();		
	int getCurrentMaterialKey();

	void setLightKey(int key);
	void setMaterialKey(int key);
	
		
	void setAmbientColors(Color groundColor, Color skyColor, Vector3 hemiAxis);
		
	void setNumMaps(int color, int normal, int gloss, int emi);
		
	void setIdColorMap (int id);
	void setIdNormalMap(int id);
	void setIdGlossMap(int id);
	void setIdEmiMap(int id);

	int getLocationTangent();
				
	/* activate the fixed OpenGL pipeline */
	void activateFixedPipeline();
		
	bool compilePerFragLight(int minDirLights, int maxDirLights,
							 int minSpotLights, int maxSpotLights,
							 int minPointLights, int maxPointLights,
							 std::set<int> matKeys);
	
		
	void activateMaterialShader(int lightKey, int materialKey, shaderType type);
	
	bool compileSky();
	void activateSky();
		
	SkyShader* getSkyShader();
		
protected:
	shaderType currentShaderType;
	GLuint currentShaderId;
		
	MaterialShader* currentMaterialShader;
		
	//for limited shader only
	/*	the first key represents the lights :
	 		nbLightsPoint + 8*nbLightsSpot + 8*8*nbLightsDir
	 	the second key represents the material :
			nbColorMaps + nbNormalMaps * 4 + nbGlossMaps * 4*4 + nbEmiMaps *4*4*4
	*/
	std::map<std::pair<int,int>,PerFragLightShader*> perFragLightPrograms;
	int currentLightKey;
	int currentMaterialKey;			
		
	SkyShader *sky;

		
};


extern ShaderManager *shaderMana;


#endif // _SHADERMANAGER_H_
